import {
  __spreadProps,
  __spreadValues,
  extractParams,
  splitParameters
} from "./chunk-ON4H4A66.js";

// src/index.ts
var Parser = class {
  constructor(params) {
    this.params = params;
    this.validActions = this.getValidActions();
    this.commandActionRegExp = this.createCommandActionRegExp();
    this.parsedInputCommands = this.getParsedInputCommands();
    this.validateDuplicateCommandExists();
  }
  parse(message) {
    const parsedValue = this.parseMessage(message, "user");
    if (parsedValue.kind === "plain") {
      return parsedValue;
    }
    const [matchedParsedInputCommand] = this.getMatchedParsedInputCommands(parsedValue);
    if (!matchedParsedInputCommand) {
      return this.createPlainMessage(message);
    }
    return __spreadProps(__spreadValues({}, parsedValue), {
      matched: matchedParsedInputCommand.command
    });
  }
  validateDuplicateCommandExists() {
    const duplicateCommands = [];
    this.params.messageItems.forEach((messageItem) => {
      const messageCommand = this.parseMessage(messageItem.command, "system");
      if (messageCommand.kind === "plain") {
        return;
      }
      const matchedParsedInputCommands = this.getMatchedParsedInputCommands(messageCommand);
      if (matchedParsedInputCommands.length > 1) {
        duplicateCommands.push({
          command: messageItem.command,
          another: matchedParsedInputCommands.filter((item) => item.command !== messageItem.command).map((item) => item.command)
        });
      }
      if (duplicateCommands.length > 0) {
        const errorMessageBodies = duplicateCommands.map((duplicateCommand) => {
          const texts = [`Command: "${duplicateCommand.command}"`, ...duplicateCommand.another.map((command) => `- "${command}"`)];
          return texts.join("\n");
        });
        const errorMessage = ["Duplicate command definitions exist.", "", ...errorMessageBodies].join("\n");
        throw new Error(errorMessage);
      }
    });
  }
  getValidActions() {
    const actions = [];
    this.params.messageItems.forEach(({ command }) => {
      const matched = command.match(/^(?<action>\w+)/);
      if (matched) {
        const action = (matched.groups || {}).action;
        action && actions.push(action);
      }
    });
    return Array.from(new Set(actions));
  }
  getParsedInputCommands() {
    const parsedInputCommands = [];
    this.params.messageItems.forEach(({ command }) => {
      const result = this.parseMessage(command, "system");
      if (result.kind === "plain") {
        return;
      }
      const parsedInputCommand = {
        command,
        action: result.action,
        actionTargets: result.actionTargets
      };
      if (result.parameters) {
        parsedInputCommand.parameters = result.parameters;
      }
      parsedInputCommands.push(parsedInputCommand);
    });
    return parsedInputCommands;
  }
  createCommandActionRegExp() {
    const actions = this.validActions.join("|");
    const pattern = `^(?<action>${actions})[\\s+]?(?<actionTargets>([\\w\\-]+\\s?)+)?`;
    return new RegExp(pattern);
  }
  createParsedParameters(paramsStrings, type) {
    const parameters = {};
    for (let i = 0; i < paramsStrings.length; i += 2) {
      const { key, value } = extractParams(paramsStrings[i], type);
      parameters[key] = value;
    }
    return parameters;
  }
  getMatchedParsedInputCommands(commandMessage) {
    return this.parsedInputCommands.filter((parsedInputCommand) => {
      const isActionMatch = parsedInputCommand.action === commandMessage.action;
      const isActionTargetMatch = JSON.stringify(parsedInputCommand.actionTargets) === JSON.stringify(commandMessage.actionTargets);
      return isActionMatch && isActionTargetMatch;
    });
  }
  parseMessage(message, type) {
    const { firstMessage, paramsMessage } = splitParameters(message, type);
    const matched = firstMessage.match(this.commandActionRegExp);
    if (!matched) {
      return this.createPlainMessage(message);
    }
    const groups = matched.groups || {};
    const commandMessage = {
      kind: "command",
      action: groups.action,
      actionTargets: (groups.actionTargets || "").split(/\s/).filter(Boolean)
    };
    if (paramsMessage.length > 0) {
      commandMessage.parameters = this.createParsedParameters(paramsMessage, type);
    }
    return commandMessage;
  }
  createPlainMessage(message) {
    return {
      kind: "plain",
      message
    };
  }
};
export {
  Parser
};
//# sourceMappingURL=index.js.map